@@ -3,9 +3,9 @@
 ;; Author: Vitalie Spinu
 ;; Maintainer: Vitalie Spinu
 ;; Copyright (C) 2013-2018 Vitalie Spinu
-;; Package-Version: 0.2.2
-;; Package-Revision: 51ffeb6ec45d
-;; Package-Requires: ((emacs "25") (polymode "0.2.2") (poly-markdown "0.2.2") (poly-noweb "0.2.2"))
+;; Package-Version: 20250502.1525
+;; Package-Revision: fee0b6e99943
+;; Package-Requires: ((emacs "25") (ess "25") (polymode "0.2.2") (poly-markdown "0.2.2") (poly-noweb "0.2.2"))
 ;; URL: https://github.com/polymode/poly-R
 ;; Keywords: languages, multi-modes
 ;;
@@ -49,7 +49,7 @@
                :hostmode 'pm-host/R
                :innermodes '(pm-inner/fundamental))
   "Root polymode with R host intended to be inherited from."
-  :group 'polymodes
+  :group 'poly-R
   :type 'object)
 
 
@@ -82,9 +82,17 @@ templates."
   :type '(repeat string)
   :group 'poly-R)
 
+(defcustom poly-r-can-eval-in-background t
+  "Whether poly-R can use background commands.
+This is similar to `ess-can-eval-in-background' but limited to
+poly-R."
+  :type 'boolean
+  :group 'poly-R)
+
+
 (define-innermode poly-r-markdown-inline-code-innermode poly-markdown-inline-code-innermode
   :mode 'ess-r-mode
-  :head-matcher (cons "[^`]\\(`{?[rR]}?\\)[ \t]" 1))
+  :head-matcher (cons "\\(?:^\\|[^`]\\)\\(`r[ #]\\)" 1))
 
 (define-obsolete-variable-alias 'pm-poly/markdown+R 'poly-markdown+r-polymode "v0.2")
 (define-obsolete-variable-alias 'pm-poly/markdown+r 'poly-markdown+r-polymode "v0.2")
@@ -96,17 +104,13 @@ templates."
 ;;;###autoload (autoload 'poly-markdown+r-mode "poly-R")
 (define-polymode poly-markdown+r-mode poly-markdown-mode :lighter " PM-Rmd"
   :innermodes '(:inherit poly-r-markdown-inline-code-innermode))
+;;;###autoload (autoload 'poly-gfm+r-mode "poly-R")
+(define-polymode poly-gfm+r-mode poly-markdown+r-mode 
+  :lighter " PM-Rmd(gfm)"
+  :hostmode 'poly-gfm-hostmode)
 
-(defun poly-r-rmarkdown-templates (&optional proc)
-  (let* ((ess-dialect "R")
-         (proc (or proc (ess-get-next-available-process "R" t)))
-         (user-dirs (if poly-r-rmarkdown-template-dirs
-                        (format "c(\"%s\")"
-                                (mapconcat #'identity
-                                           poly-r-rmarkdown-template-dirs
-                                           "\", \""))
-                      "c()"))
-         (cmd (concat "
+(defvar poly-r--rmarkdown-template-command
+  "
 local({
 list_templates <-
  function(user_dirs) {
@@ -121,7 +125,7 @@ list_templates <-
             yaml <- yaml::yaml.load(readLines(yaml_file, encoding = 'UTF-8'))
             if (is.null(desc <- yaml$description)) desc <- 'nil'
             if (!is.null(pkg)) d <- basename(d)
-            cat(sprintf('(\"%s\" \"%s\" \"%s\" \"%s\")\n', pkg_name, d, yaml$name, desc))
+            cat(sprintf('(\"%%s\" \"%%s\" \"%%s\" \"%%s\")\n', pkg_name, d, yaml$name, desc))
           }
         }
       }
@@ -131,7 +135,7 @@ list_templates <-
       files <- list.files(user_dir,  '\\\\.[Rr]md$', full.names = TRUE)
       for (f in files) {
         name <- sub('\\\\.[^.]+$','', basename(f))
-        cat(sprintf('(\"USER\" \"%s\" \"%s\")\n', f, name))
+        cat(sprintf('(\"USER\" \"%%s\" \"%%s\")\n', f, name))
       }
       list_from_dir(user_dir)
     }
@@ -142,9 +146,19 @@ list_templates <-
     }
     cat(')\n')
   }\n
-"
-                      (format "list_templates(%s)})\n" user-dirs))))
-    (with-current-buffer (ess-command cmd nil nil nil nil proc)
+ list_templates(%s)})\n")
+
+(defun poly-r-rmarkdown-templates (&optional proc timeout)
+  (let* ((ess-dialect "R")
+         (proc (or proc (ess-get-next-available-process "R" t)))
+         (user-dirs (if poly-r-rmarkdown-template-dirs
+                        (format "c(\"%s\")"
+                                (mapconcat #'identity
+                                           poly-r-rmarkdown-template-dirs
+                                           "\", \""))
+                      "c()"))
+         (cmd (format poly-r--rmarkdown-template-command user-dirs)))
+    (with-current-buffer (ess-command cmd nil nil nil nil proc nil timeout)
       (goto-char (point-min))
       (if (save-excursion (re-search-forward "\\+ Error" nil t))
           (error "%s" (buffer-string))
@@ -156,7 +170,8 @@ list_templates <-
   (let* ((proc (ess-get-next-available-process "R" t))
          (templates (process-get proc :rmarkdown-templates)))
     (unless templates
-      (setq templates (poly-r-rmarkdown-templates proc))
+      (setq templates (ignore-errors
+                        (poly-r-rmarkdown-templates proc 0.2)))
       (when (< 2000 (length templates))
         (process-put proc :rmarkdown-templates templates)))
     (mapcar (lambda (el)
@@ -192,21 +207,21 @@ list_templates <-
 (defvar poly-r--rmarkdown-create-from-template-hist nil)
 (defun poly-r-rmarkdown-create-from-template ()
   "Create new Rmarkdown project from template.
-Templates are provided by R packages in the format described in
-the help of 'draft' function and chapter 17 of the RMarkdown
-book. This function also searches for templates in
-`poly-r-rmarkdown-template-dirs' directories. Templates in those
-directories are either simple Rmd files or template directories
-with template.yaml metadata as enforced by rmarkdwon template
-format.
+  Templates are provided by R packages in the format described in
+  the help of 'draft' function and chapter 17 of the RMarkdown
+  book. This function also searches for templates in
+  `poly-r-rmarkdown-template-dirs' directories. Templates in those
+  directories are either simple Rmd files or template directories
+  with template.yaml metadata as enforced by rmarkdwon template
+  format.
 
-Common packages containing templates are rticles, tufte,
-xaringan, prettydoc, revealjs, flexdashboards.  Some more
-templates at:
+  Common packages containing templates are rticles, tufte,
+  xaringan, prettydoc, revealjs, flexdashboards.  Some more
+  templates at:
 
- https://github.com/mikey-harper/example-rmd-templates
- https://github.com/hrbrmstr/markdowntemplates
- https://github.com/svmiller/svm-r-markdown-templates"
+  https://github.com/mikey-harper/example-rmd-templates
+  https://github.com/hrbrmstr/markdowntemplates
+  https://github.com/svmiller/svm-r-markdown-templates"
   (interactive)
   (let* ((specs (poly-r-rmarkdown-templates))
          (spec (cdr (pm--completing-read "Template: "
@@ -221,7 +236,10 @@ templates at:
   "Menu for poly-r+markdown-mode"
   '("RMarkdown"
     ("Templates"
-     :active (ess-get-next-available-process "R" t)
+     ;; TODO: Use `ess-get-next-available-bg-process' after release
+     :active (and poly-r-can-eval-in-background
+                  ess-can-eval-in-background
+                  (ess-get-next-available-process "R" t))
      :filter poly-r-rmarkdown-templates-menu)))
 
 (define-key poly-markdown+r-mode-map (kbd "M-n M-m") #'poly-r-rmarkdown-create-from-template)
@@ -404,6 +422,12 @@ templates at:
     (command "Rscript -e \"rmarkdown::render('%i', output_format = 'all')\"")
     (output-file #'pm--rmarkdown-output-file-sniffer)))
 
+(defun pm--rmarkdown-shell-default-selector (action &rest _ignore)
+  (cl-case action
+    (doc "DEFAULT")
+    (command "Rscript -e \"rmarkdown::render('%i', output_format = NULL)\"")
+    (output-file #'pm--rmarkdown-output-file-sniffer)))
+
 (define-obsolete-variable-alias 'pm-exporter/Rmarkdown 'poly-r-markdown-exporter "v0.2")
 (defcustom poly-r-markdown-exporter
   (pm-shell-exporter :name "Rmarkdown"
@@ -412,6 +436,7 @@ templates at:
                         "Rscript -e \"rmarkdown::render('%i', output_format = '%t', output_file = '%o')\""))
                      :to
                      '(("auto" . pm--rmarkdown-shell-auto-selector)
+                       ("default" . pm--rmarkdown-shell-default-selector)
                        ("html" "html" "html document" "html_document")
                        ("pdf" "pdf" "pdf document" "pdf_document")
                        ("word" "docx" "word document" "word_document")
@@ -420,10 +445,10 @@ templates at:
                        ("slidy" "html" "slidy presentation" "slidy_presentation")
                        ("beamer" "pdf" "beamer presentation" "beamer_presentation")))
   "R Markdown exporter.
-Please not that with 'AUTO DETECT' export options, output file
-names are inferred by Rmarkdown from YAML description
-block. Thus, output file names don't comply with
-`polymode-exporter-output-file-format'."
+  Please not that with 'AUTO DETECT' export options, output file
+  names are inferred by Rmarkdown from YAML description
+  block. Thus, output file names don't comply with
+  `polymode-exporter-output-file-format'."
   :group 'polymode-export
   :type 'object)
 
@@ -434,6 +459,14 @@ block. Thus, output file names don't comply with
     (command "rmarkdown::render('%I', output_format = 'all', knit_root_dir=getwd())")
     (output-file #'pm--rmarkdown-output-file-from-.Last.value)))
 
+(defun pm--rmarkdown-callback-default-selector (action &rest _ignore)
+  (cl-case action
+    (doc "DEFAULT")
+    ;; last file is not auto-detected unless we cat new line
+    (command "rmarkdown::render('%I', output_format = NULL, knit_root_dir=getwd())")
+    (output-file #'pm--rmarkdown-output-file-from-.Last.value)))
+
+
 (define-obsolete-variable-alias 'pm-exporter/Rmarkdown-ESS 'poly-r-markdown-ess-exporter "v0.2")
 (defcustom poly-r-markdown-ess-exporter
   (pm-callback-exporter :name "Rmarkdown-ESS"
@@ -444,6 +477,7 @@ block. Thus, output file names don't comply with
                         '(("auto" . pm--rmarkdown-callback-auto-selector)
                           ("html" "html" "html document" "html_document")
                           ("pdf" "pdf" "pdf document" "pdf_document")
+                          ("default" . pm--rmarkdown-callback-default-selector)
                           ("word" "docx" "word document" "word_document")
                           ("md" "md" "md document" "md_document")
                           ("ioslides" "html" "ioslides presentation" "ioslides_presentation")
@@ -452,10 +486,10 @@ block. Thus, output file names don't comply with
                         :function 'pm--ess-run-command
                         :callback 'pm--ess-callback)
   "R Markdown exporter.
-Please not that with 'AUTO DETECT' export options, output file
-names are inferred by Rmarkdown from YAML description
-block. Thus, output file names don't comply with
-`polymode-exporter-output-file-format'."
+  Please not that with 'AUTO DETECT' export options, output file
+  names are inferred by Rmarkdown from YAML description
+  block. Thus, output file names don't comply with
+  `polymode-exporter-output-file-format'."
   :group 'polymode-export
   :type 'object)
 
@@ -533,7 +567,7 @@ block. Thus, output file names don't comply with
                      :to
                      '(("html" "html" "Shiny Web App")))
   "Shiny exporter of Rmd documents in stand alone shell.
-The Rmd yaml preamble must contain runtime: shiny declaration."
+  The Rmd yaml preamble must contain runtime: shiny declaration."
   :group 'polymode-export
   :type 'object)
 
@@ -546,7 +580,7 @@ The Rmd yaml preamble must contain runtime: shiny declaration."
                         '(("html" "html" "Shiny Web App"))
                         :function 'pm--ess-run-command)
   "Shiny exporter of Rmd documents within ESS process.
-The Rmd yaml preamble must contain runtime: shiny declaration."
+  The Rmd yaml preamble must contain runtime: shiny declaration."
   :group 'polymode-export
   :type 'object)
 
@@ -561,7 +595,7 @@ The Rmd yaml preamble must contain runtime: shiny declaration."
                    :from-to
                    '(("latex" "\\.\\(r?tex\\|rnw\\)\\'" "tex" "LaTeX" "Rscript -e \"knitr::knit('%i', output='%o')\"")
                      ("html" "\\.r?x?html?\\'" "html" "HTML" "Rscript -e \"knitr::knit('%i', output='%o')\"")
-                     ("markdown" "\\.r?md]\\'" "md" "Markdown" "Rscript -e \"knitr::knit('%i', output='%o')\"")
+                     ("markdown" "\\.[rR]?md\\'" "md" "Markdown" "Rscript -e \"knitr::knit('%i', output='%o')\"")
                      ("rst" "\\.rst" "rst" "ReStructuredText" "Rscript -e \"knitr::knit('%i', output='%o')\"")
                      ("brew" "\\.r?brew\\'" "brew" "Brew" "Rscript -e \"knitr::knit('%i', output='%o')\"")
                      ("asciidoc" "\\.asciidoc\\'" "txt" "AsciiDoc" "Rscript -e \"knitr::knit('%i', output='%o')\"")
@@ -580,7 +614,7 @@ The Rmd yaml preamble must contain runtime: shiny declaration."
                       :from-to
                       '(("latex" "\\.\\(r?tex\\|rnw\\)\\'" "tex" "LaTeX" "knitr::knit('%I', output='%O')")
                         ("html" "\\.[xX]?html?\\'" "html" "HTML" "knitr::knit('%I', output='%O')")
-                        ("markdown" "\\.r?md]\\'" "md" "Markdown" "knitr::knit('%I', output='%O')")
+                        ("markdown" "\\.[rR]?md\\'" "md" "Markdown" "knitr::knit('%I', output='%O')")
                         ("rst" "\\.rst\\'" "rst" "ReStructuredText" "knitr::knit('%I', output='%O')")
                         ("brew" "\\.r?brew]\\'" "brew" "Brew" "knitr::knit('%I', output='%O')")
                         ("asciidoc" "\\.r?asciidoc\\'" "txt" "AsciiDoc" "knitr::knit('%I', output='%O')")
